package benefactoraddressvalidator

import (
	"context"
	"fmt"

	param "git.gocasts.ir/ebhomengo/niki/param/benefactor/benefactore"
	errmsg "git.gocasts.ir/ebhomengo/niki/pkg/err_msg"
)

type BenefactorSvc interface {
	BenefactorExistByID(ctx context.Context, request param.BenefactorExistByIDRequest) (param.BenefactorExistByIDResponse, error)
}
type Repository interface {
	IsExistCityByID(ctx context.Context, id uint) (bool, error)
	IsExistProvinceByID(ctx context.Context, id uint) (bool, error)
}
type Validator struct {
	benefactorSvc BenefactorSvc
	repository    Repository
}

func New(benefactorSvc BenefactorSvc, repository Repository) Validator {
	return Validator{benefactorSvc: benefactorSvc, repository: repository}
}

func (v Validator) doesBenefactorExist(value interface{}) error {
	benefactorID, ok := value.(uint)
	if !ok {
		return fmt.Errorf(errmsg.ErrorMsgSomethingWentWrong)
	}
	_, err := v.benefactorSvc.BenefactorExistByID(context.Background(), param.BenefactorExistByIDRequest{ID: benefactorID})
	if err != nil {
		return fmt.Errorf(errmsg.ErrorMsgNotFound)
	}
	// TODO: check benefactor ID given from user most check with claims (benefactorID)
	return nil
}

func (v Validator) doesProvinceExist(value interface{}) error {
	provinceID, ok := value.(uint)
	if !ok {
		return fmt.Errorf(errmsg.ErrorMsgSomethingWentWrong)
	}
	isExisted, err := v.repository.IsExistProvinceByID(context.Background(), provinceID)
	if err != nil {
		return fmt.Errorf(errmsg.ErrorMsgNotFound)
	}
	if !isExisted {
		return fmt.Errorf(errmsg.ErrorMsgNotFound)
	}

	return nil
}

func (v Validator) doesCityExist(value interface{}) error {
	cityID, ok := value.(uint)
	if !ok {
		return fmt.Errorf(errmsg.ErrorMsgSomethingWentWrong)
	}
	isExisted, err := v.repository.IsExistCityByID(context.Background(), cityID)
	if err != nil {
		return fmt.Errorf(errmsg.ErrorMsgNotFound)
	}
	if !isExisted {
		return fmt.Errorf(errmsg.ErrorMsgNotFound)
	}

	return nil
}
